#include <Wire.h>
#include <ds3231.h>
#include "RTClib.h"
#include "Adafruit_MCP23008.h"
#include <OneWire.h> 
#include <DallasTemperature.h>

#define ONE_WIRE_BUS 6 

Adafruit_MCP23008 mcp1, mcp2, mcp3, mcp4;

RTC_DS3231 rtc;

DateTime now;

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

struct ts clock;

int count = 0;
const long TEMPO_LAMPEGGIO = 1000;  //Tempo lampeggio del punto su secondo display
const long TEMPO_CAMBIO = 5000;  //Tempo lampeggio del punto su secondo display
int Tipo_Dato = 0;
long cambio = 0;


//********************************************************** S E T U P **********************************************************
void setup() {
  Serial.begin(9600);
  rtc.begin();
  sensors.begin(); 
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  DS3231_init(DS3231_CONTROL_INTCN);

  mcp1.begin(0x20);
  mcpPinMode(mcp1);

  mcp2.begin(0x21);
  mcpPinMode(mcp2);

  mcp3.begin(0x22);
  mcpPinMode(mcp3);

  mcp4.begin(0x23);
  mcpPinMode(mcp4);

  Test_Display();
}


//*********************************************************** L O O P ***********************************************************
void loop()
{
  if (rtc.lostPower()) 
  {
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }
  
  
  now = rtc.now();            //lettura dell'ora aggiornata
  clock.min = now.minute();   //esporto i minuti
  clock.hour = now.hour();    //esportole ore
  clock.sec = now.second();   //esporto i secondi

  DS3231_set(clock);
  //Dati visualizzati su seriale come debug
  Serial.print("ora: ");
  Serial.print(clock.hour);
  Serial.print(" : ");
  Serial.print(clock.min);
  Serial.print(" : ");
  Serial.print(clock.sec);
  Serial.print("\n");

  if ((millis()-TEMPO_CAMBIO)>=cambio)
  {
    if (Tipo_Dato==0) Tipo_Dato = 1;
    else Tipo_Dato = 0;
    cambio=millis();
  }

  //Tipo_Dato = 1;
  switch(Tipo_Dato)
  {
    case 0:
      LedAcceso(decine(clock.hour), mcp1);//visulizzo le decine dell'ora
      LedAcceso(unita(clock.hour), mcp2); //visualizzo le unità dell'ora
      LedAcceso(decine(clock.min), mcp3); //visulizzo le decine dei minuti
      LedAcceso(unita(clock.min), mcp4);  //visualizzo le unità dei minuti
      punto_lampeggiante(mcp2,TEMPO_LAMPEGGIO);      //faccio lampeggiare il punto del secondo display ogni secondo
      break;
      
    case 1:
      Visualizza_Temperatura();
      break;
      
    default:
      break;
  }

  delay(200);
}


//********************************************************** T E S T _ D I S P L A Y **********************************************************
//Accende in sequenza i numeri da 0 a 9, incluso il punto del display per testare il display e la comunicazione I2C
void Test_Display()
{
  punto(mcp1,HIGH); 
  punto(mcp2,HIGH); 
  punto(mcp3,HIGH); 
  punto(mcp4,HIGH); 
  for(int i=0;i<10;i++)
  {
    LedAcceso(i, mcp1); //ore decine
    LedAcceso(i, mcp2); //ore unità  
    LedAcceso(i, mcp3); //minuti decine
    LedAcceso(i, mcp4); //minuti unità    
    delay(500);
  }   
  LedAcceso(10, mcp1); //ore decine
  LedAcceso(10, mcp2); //ore unità  
  LedAcceso(10, mcp3); //minuti decine
  LedAcceso(10, mcp4); //minuti unità
  delay(1000);
}

//********************************************************** L E G G I _ T E M P E R A T U R A **********************************************************
String Leggi_Temperatura()
{
  String Temperature;
  sensors.requestTemperatures(); // Send the command to get temperature readings 
  Temperature = String(sensors.getTempCByIndex(0)); // Why "byIndex"?   
  return Temperature;
}



void Visualizza_Temperatura()
{
  String Val = Leggi_Temperatura();
  Serial.print("Temp: ");
  Serial.println(Val);
  punto(mcp1, LOW);
  punto(mcp2, LOW);
  punto(mcp3, LOW);
  punto(mcp4, LOW);
  if (Val.charAt(0)=='-')   //Se la temperatura è negativa
  {
    LedAcceso(11, mcp1);//visulizzo le decine dell'ora        
    if (Val.charAt(2)=='.')
    {
      LedAcceso(0, mcp2); //visualizzo le unità dell'ora
      LedAcceso(int(Val.charAt(1))-48, mcp3); //visulizzo le decine dei minuti
    }
    else
    {
      LedAcceso(int(Val.charAt(1))-48, mcp2); //visualizzo le unità dell'ora
      LedAcceso(int(Val.charAt(2))-48, mcp3); //visulizzo le decine dei minuti
    }    
    LedAcceso(12, mcp4); //visulizzo il pallino dei gradi
  }
  else //Se la temperatura è positiva
  {        
    LedAcceso(10, mcp1);//visulizzo le decine dell'ora
    if (Val.charAt(1)=='.')
    {
      LedAcceso(0, mcp2); //visualizzo le unità dell'ora
      LedAcceso(int(Val.charAt(1))-48, mcp3); //visulizzo le decine dei minuti
      LedAcceso(12, mcp3); //visulizzo il pallino dei gradi
    }
    else
    {
      LedAcceso(int(Val.charAt(0))-48, mcp2); //visualizzo le unità dell'ora
      LedAcceso(int(Val.charAt(1))-48, mcp3); //visulizzo le decine dei minuti      
    }
    LedAcceso(12, mcp4); //visulizzo il pallino dei gradi
  }       
}
